#Applying Outline Analyses in R
#Input data set:
#	-Images in the ppm format, possibly binarized
#Further Reading: Claude, J. (2008) "Morphometrics with R". Gentleman, R., Hornik, K., and...
#			Parmigiani, G. (eds) "Use R!", vol. ii, 316 pp. (Springer).
#Disclaimer: Most of the functions in this sheet are at least based on functions developed by...
#	Claude (2008), and this publication should thus always be cited when using the code...
#	provided here. References to pages in that book are given in parentheses wherever necessary.
#Author: Manuel Weinkauf  (mweinkauf@marum.de)
#Version: 1.3
#Date: 4 July 2014

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
#This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.#
#To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/3.0/.                   #
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#

#**************************************************************************************
#Setting working directory
#setwd("C:/R_TestData/EFA")

#########################################################################
# Image conversion							#
# Necessary programs: Image Magic					#
# Necessary input variables:						#
#		ImageName: Name part of images (same for all images)	#
#			*string*					#
#		StartNum: Smallest of row of continuous numbers...	#
#			used to number images				#
#			*integer*					#
#		StopNum: Largest of row of continuous numbers...	#
#			used to number images				#
#			*integer*					#
#		Scaling: relative scaling of the image during conversion#
#			*integer*					#
#			default=100					#
#		ImageType: Image type as ".xyz"				#
#			*string*					#
#		OutputType: Image type for output as ".xyz"		#
#			*string*					#
# Output data: images in OutputType format				#
# Input dataset: images in ImageType format				#
#########################################################################

ImageConversion<-function (ImageName, StartNum, StopNum, Scaling=100, ImageType, OutputType) {
	for (i in (StartNum:StopNum)) {
		com<-paste("convert ", ImageName, i, ImageType, " ", "-resize ", Scaling, "%", " ", ImageName, i, OutputType, sep="")
		shell(com)
	}
}

#########################################################################
# Outline rastering (p. 47)						#
# Necessary input variables:						#
#		x: manually chosen starting point			#
#			*integer*					#
#		imagematrix: name of image to use			#
#			*string*					#
# Output data: list of x-y coordinates of all pixel along outline	#
# Input dataset: image in ppm format					#
#########################################################################

Conte<-function (x, imagematrix) {
	#Reading image as image matrix and calculate dimensions
	I<-imagematrix
	x<-rev(x)
	x[1]<-dim(I)[1]-x[1]
	
	#Going from the starting point to the left, until reaching a pixel that significantly differs in grey value from the pixel next to it
	while (abs(I[x[1], x[2]]-I[x[1], (x[2]-1)])<0.1) {x[2]<-x[2]-1}
	#Defining that pixel as starting point of the outline
	a<-1
	
	#Writing indices of eight pixels surrounding starting position (0,0) into matrix
	M<-matrix(c(0, -1, -1, -1, 0, 1, 1, 1, 1, 1, 0, -1, -1, -1, 0, 1), 2, 8, byrow=TRUE)
	#Attaching first and last column of matrix as ninth and tenth column on matrix
	M<-cbind(M[,8], M, M[,1])
	
	#Initialising parameters
	X<-0
	Y<-0
	x1<-x[1]
	x2<-x[2]
	SS<-NA
	S<-6
	
	#Allocating all pixels that belong to the outline (running clockwise)
	while ((any(c(X[a], Y[a])!=c(x1, x2)) | length(X)<3)) {
		if(abs(I[x[1]+M[1,S+1], x[2]+M[2,S+1]]-I[x[1], x[2]])<0.1) {a<-a+1; X[a]<-x[1]; Y[a]<-x[2]; x<-x+M[,S+1]; SS[a]<-S+1; S<-(S+7)%%8}
		else if (abs(I[x[1]+M[1,S+2], x[2]+M[2,S+2]]-I[x[1], x[2]])<0.1) {a<-a+1; X[a]<-x[1]; Y[a]<-x[2]; x<-x+M[,S+2]; SS[a]<-S+2; S<-(S+7)%%8}
		else if (abs(I[x[1]+M[1,(S+3)], x[2]+M[2,(S+3)]]-I[x[1], x[2]])<0.1) {a<-a+1; X[a]<-x[1]; Y[a]<-x[2]; x<-x+M[,(S+3)]; SS[a]<-S+3; S<-(S+7)%%8}
		else S<-(S+1)%%8
	}
	list(X = (Y[-1]), Y = ((dim(I)[1]-X))[-1])
}

#########################################################################
# Curvelinear fitting (p. 52)						#
# Necessary input variables:						#
#		Input: Raw pixel coordinates of outline as extracted...	#
#			with function Conte				#
#			*vector*					#
#		n: Desired number of curvelinearily equally spaced...	#
#			points along outline				#
#			*integer*					#
# Output data: matrix with equally spaced coordinates of outline	#
# Input dataset: vector of raw x and y coordinates			#
#########################################################################

EquiDist<-function (Input, n) {
	RcEqX<-(Input$X[seq(1, length(Input$X), length=(n+1))])[-1]
	RcEqY<-(Input$Y[seq(1, length(Input$Y), length=(n+1))])[-1]
	X_Temp<-as.matrix(RcEqX)
	Y_Temp<-as.matrix(RcEqY)
	M<-cbind(X_Temp, Y_Temp)
	colnames(M)<-c("X", "Y")
	rownames(M)<-paste("Coord", 1:n, sep="")
	M
}

#########################################################################
# Smoothing outline (p. 55)						#
# Necessary input variables:						#
#		M: matrix of equally spaced outline points		#
#			*matrix*					#
#		n: number of iterations					#
#			*integer*					#
# Output data: list of smoothed x-y coordinates of outline		#
# Input dataset: curvelinear equally spaced x-y coordinates of outline	#
#########################################################################

smoothout<-function(M, n){
	p<-dim(M)[1]
	a<-0
	while(a<=n){
		a<-a+1
		Ms<-rbind(M[p,], M[-p,])
		Mi<-rbind(M[-1,], M[1,])
		M<-M/2+Ms/4+Mi/4
	}
	M
}

#########################################################################
# Outline extraction							#
# Necessary packages: pixmap						#
# Necessary functions: Conte, EquiDist, smoothout			#
# Necessary input variables:						#
#		ImageName: Name part of images (same for all images)	#
#			*string*					#
#		StartNum: Smallest of row of continuous numbers...	#
#			used to number images				#
#			*integer*					#
#		StopNum: Largest of row of continuous numbers...	#
#			used to number images				#
#			*integer*					#
#		OutlinePoints: Desired number of equidistantly spaced...#
#			points along outline (see function EquiDist)	#
#			*integer*					#
#			default=100					#
#		Smoothing: Desired number of iterations for outline...	#
#			smoothing (see function smoothout)		#
#			*integer*					#
#			default=1					#
#		RawVersion: Defines whether or not the raw (i.e. un-...	#
#			smoothed version of the outline coordinates...	#
#			should be exported as well.			#
#			*logical*					#
#			TRUE=Export raw version				#
#			FALSE=Do not export raw version			#
#			default=FALSE					#
# Output data: matrix containing x and y coordinates of specified...	#
#			number of equidistant points along outline	#
# Input dataset: several logically named image files in .ppm format	#
#########################################################################

#Loading packages
library(pixmap)

OutlineExtraction<-function (ImageName, StartNum, StopNum, OutlinePoints=100, Smoothing=1, RawVersion=FALSE) {
	#Setting up control matrix
	ExtFail<-matrix(NA, (StopNum-StartNum)+1, 1)
	colnames(ExtFail)<-c("Success")
	rownames(ExtFail)<-paste(ImageName, c(StartNum:StopNum), sep="_")
	#Start data acquisition
	MatPos<-1
	for (i in StartNum:StopNum) {
		print(i)
		Image<-paste(ImageName, i, ".ppm", sep="")
		#Reading image
		y<-read.pnm(Image)
		#Converting image to grey scale
		y<-as(y, "pixmapGrey")
		#Converting greyscale to grey on white
		y@grey[which(y@grey>=0.9)]<-1
		y@grey[which(y@grey<0.9)]<-0.7
		
		#Setting margins and plotting image
		par(mar=c(1, 1, 1, 1))
		plot(y)
		
		#Manually chosing starting point and decide whether or not outline is well fitted
		cont<-NA
		while(any(is.na(cont), cont=="n", (cont!="y" && cont!="c"))) {
			print("Click within the object to the right of the starting point!")
			flush.console()
			start<-locator(1)
			Rc<-Conte(c(round(start$x), round(start$y)), y@grey)
			lines(Rc$X, Rc$Y, lwd=4)
			arrows(0, Rc$Y[1], Rc$X[1], Rc$Y[1], length=0.1)
			bringToTop(-1)
			cont<-readline("Is the outline correct? (y=proceed,n=try again,c=cancel and proceed)")
			if (cont!="y" && cont!="c") {par(mar=c(1, 1, 1, 1)); plot(y)}
		}
		{if (cont=="y") {ExtFail[MatPos,1]<-1}
			else {ExtFail[MatPos,1]<-0}
		}

		#Normalising curve for curvelinear equally spaced number of points
		RcEqual<-EquiDist(Rc, OutlinePoints)
		
		#Smoothing outline
		RcSmooth<-smoothout(RcEqual, Smoothing)
		
		#Writing x-y coordinates of outline into file
		if (RawVersion==TRUE) {File<-paste(ImageName, i, "_Raw.txt", sep=""); write.table(RcEqual, File, sep="\t")}
		File<-paste(ImageName, i, ".txt", sep="")
		write.table(RcSmooth, File, sep="\t")
		
		MatPos<-MatPos+1
	}
	#Save list of successes, i.e. in which specimens did outline extraction fail
	write.table(ExtFail, "OutlineSuccess.txt", sep="\t")
}

#########################################################################
# Performing Zahn--Roskies Fourier Analysis (pp. 218f)			#
# Necessary input variables:						#
#		Input: Digitized outline points coordinates		#
#			*matrix*					#
#		Harmonics: Desired number of harmonics to be computed	#
#			*integer*					#
# Output data: list containing harmonics (ao, an, bn), variation of...	#
#			tangent angle (phi), distances along peri...	#
#			meter in radians (t), perimeter of outline,...	#
#			and first tangent angle thetao			#
# Input dataset: list of equally spaced x-y coordinates along...	#
#			digitized outline				#
#########################################################################

ZRfourier<-function(Input, Harmonics) {
	p<-dim(Input)[1]
	an<-numeric(Harmonics)
	bn<-numeric(Harmonics)
	tangvect<-Input-rbind(Input[p,], Input[-p,])
	perim<-sum(sqrt(apply((tangvect)^2, 1, sum)))
	v0<-(Input[1,]-Input[p,])
	tet1<-Arg(complex(real=tangvect[,1], imaginary=tangvect[,2]))
	tet0<-tet1[1]
	t1<-(seq(0, 2*pi, length=(p+1)))[1:p]
	phi<-(tet1-tet0-t1)%%(2*pi)
	ao<-2*sum(phi)/p
	for (i in 1:Harmonics) {
		an[i]<-(2/p)*sum(phi*cos(i*t1))
		bn[i]<-(2/p)*sum(phi*sin(i*t1))
	}
	list(ao=ao, an=an, bn=bn, phi=phi, t=t1, perimeter=perim, thetao=tet0)
}

#########################################################################
# Performing elliptic Fourier analysis (pp. 222f)			#
# Necessary input variables:						#
#		Input: Digitized outline points coordinates		#
#			*matrix*					#
#		Harmonics: Desired number of harmonics to be computed	#
#			*integer*					#
#			default=Number of outline points/2		#
# Output data: list containing centroid coordinates (ao, co) and...	#
#			values for four coeffecients per harmonic	#
# Input dataset: list of equally spaced x-y coordinates along...	#
#			digitized outline				#
#########################################################################

efourier<-function (Input, Harmonics=((dim(Input)[1])/2)) {
	p<-dim(Input)[1]
	Dx<-Input[,1]-Input[c(p,(1:p-1)),1]
	Dy<-Input[,2]-Input[c(p,(1:p-1)),2]
	Dt<-sqrt(Dx^2+Dy^2)
	t1<-cumsum(Dt)
	t1m1<-c(0, t1[-p])
	T<-sum(Dt)
	an<-bn<-cn<-dn<-numeric(Harmonics)
	for (i in 1:Harmonics) {
		an[i]<-(T/(2*pi^2*i^2))*sum((Dx/Dt)*(cos(2*i*pi*t1/T)-cos(2*pi*i*t1m1/T)))
		bn[i]<-(T/(2*pi^2*i^2))*sum((Dx/Dt)*(sin(2*i*pi*t1/T)-sin(2*pi*i*t1m1/T)))
		cn[i]<-(T/(2*pi^2*i^2))*sum((Dy/Dt)*(cos(2*i*pi*t1/T)-cos(2*pi*i*t1m1/T)))
		dn[i]<-(T/(2*pi^2*i^2))*sum((Dy/Dt)*(sin(2*i*pi*t1/T)-sin(2*pi*i*t1m1/T)))
	}
	ao<-2*sum(Input[,1]*Dt/T)
	co<-2*sum(Input[,2]*Dt/T)
	list(ao=ao, co=co, an=an, bn=bn, cn=cn, dn=dn)
}

#########################################################################
# Performing normalized elliptic Fourier analysis (p. 226)		#
# Necessary functions: efourier						#
# Necessary input variables:						#
#		Input: Digitized outline points coordinates		#
#			*matrix*					#
#		Harmonics: Desired number of harmonics to be computed	#
#			*integer*					#
#			default=Number of outline points/2		#
#		start: Is the position of the starting point to be...	#
#			preserved?					#
#			*logical*					#
#			TRUE=Starting point must be preserved		#
#			FALSE=Starting point can not be preserved	#
#			default=TRUE					#
# Output data: list containing values for four standardised...		#
#			coefficients (A-D) per harmonic; magnitude...	#
#			of semi-major axis of first harmonic (size);...	#
#			angle between starting point and semi-major...	#
#			axis of first harmonic (theta); orientation...	#
#			of first harmonic ellipse (psi); harmonic...	#
#			coefficients ao and co				#
# Input dataset: list of equally spaced x-y coordinates along...	#
#			digitized outline				#
#########################################################################

NEF<-function(Input, Harmonics=dim(Input)[1]/2, start=TRUE){
	ef<-efourier(Input, Harmonics)
	A1<-ef$an[1]
	B1<-ef$bn[1]
	C1<-ef$cn[1]
	D1<-ef$dn[1]
	theta<-(0.5*atan(2*(A1*B1+C1*D1)/(A1^2+C1^2-B1^2-D1^2)))%%pi
	phaseshift<-matrix(c(cos(theta), sin(theta), -sin(theta), cos(theta)), 2, 2)
	M2<-matrix(c(A1, C1, B1, D1), 2, 2)%*%phaseshift
	v<-apply(M2^2, 2, sum)
	if (v[1]<v[2]) {theta<-theta+pi/2} 
	theta<-(theta+pi/2)%%pi-pi/2
	Aa<-A1*cos(theta)+B1*sin(theta)
	Cc<-C1*cos(theta)+D1*sin(theta)
	scale<-sqrt(Aa^2+Cc^2)
	psi<-atan(Cc/Aa)
	if (Aa<0){psi<-psi+pi}
	size<-(1/scale)
	rotation<-matrix(c(cos(psi), -sin(psi), sin(psi), cos(psi)), 2, 2)
	A<-B<-C<-D<-numeric(Harmonics)
	if (start==TRUE) {theta<-0}
	for (i in 1:Harmonics) {
    		mat<-size*rotation%*%matrix(c(ef$an[i], ef$cn[i], ef$bn[i], ef$dn[i]), 2, 2)%*%matrix(c(cos(i*theta), sin(i*theta), -sin(i*theta), cos(i*theta)), 2, 2)
		A[i]<-mat[1,1]
		B[i]<-mat[1,2]
		C[i]<-mat[2,1]
		D[i]<-mat[2,2]
	}
	list(A=A, B=B, C=C, D=D, size=scale, theta=theta, psi=psi, ao=ef$ao, co=ef$co, Aa=Aa, Cc=Cc)
}

#########################################################################
# Reconstructing outline from Zahn--Roskies Fourier coefficients...	#
#		(p. 219)						#
# Necessary input variables:						#
#		ao: harmonic coefficient a0				#
#			*real*						#
#		an, bn: Set of coefficients per harmonic		#
#			*list*						#
#		Harmonics: Desired number of harmonics to be used...	#
#			for outline reconstruction			#
#			*integer*					#
#		Points: Desired number of outline points to be...	#
#			reconstructed					#
#			IMPORTANT: Due to the applied mathematics...	#
#				value must be exactly the same as...	#
#				the number of outline points...		#
#				originally extracted and used to...	#
#				perform the Z--R Fourier Analysis	#
#			*integer*					#
#		thetao: first tangent angle thetao			#
#			*real*						#
#			default=0					#
# Output dataset: list of outline points reconstructed for harmonic...	#
#			coefficients					#
# Input dataset: list containing parameters from a Zahn--Roskies...	#
#			Fourier analysis				#
#########################################################################

iZRfourier<-function(ao, an, bn, Harmonics, Points, thetao=0) {
	theta<-seq(0, 2*pi, length=Points+1)[-(Points+1)]
	harm<-matrix(NA, Harmonics, Points)
 	for (i in 1:Harmonics) {
		harm[i,]<-an[i]*cos(i*theta)+bn[i]*sin(i*theta)
	}
	phi<-(ao/2)+apply(harm, 2, sum)
	vect<-matrix(NA, 2, Points)
	Z<-complex(modulus=(2*pi)/Points, argument=phi+theta+thetao)
	Z1<-cumsum(Z)
	list(angle=theta, phi=phi, X=Re(Z1), Y=Im(Z1))
}

#########################################################################
# Reconstructing outline from elliptic Fourier coefficients (p. 223)	#
# Necessary input variables:						#
#		an, bn, cn, dn: Set of coefficients per harmonic	#
#			*list*						#
#		Harmonics: Desired number of harmonics to be used...	#
#			for outline reconstruction			#
#			*integer*					#
#		Points: Desired number of outline points to be...	#
#			reconstructed					#
#			*integer*					#
# Output dataset: list of outline points reconstructed for harmonic...	#
#			coefficients					#
# Input dataset: list containing centroid coordinates (a0, co) and...	#
#			values for four coeffecients per harmonic;...	#
#			number of harmonics to be used; number of...	#
#			points to be reconstructed			#
#########################################################################

iefourier<-function (an, bn, cn, dn, Harmonics, Points, ao=0, co=0) {
	theta<-seq(0,2*pi, length=Points+1)[-(Points+1)]
	harmx<-matrix(NA, Harmonics, Points)
	harmy<-matrix(NA, Harmonics, Points)
	for (i in 1:Harmonics) {
		harmx[i,]<-an[i]*cos(i*theta)+bn[i]*sin(i*theta)
		harmy[i,]<-cn[i]*cos(i*theta)+dn[i]*sin(i*theta)
	}
	x<-(ao/2)+apply(harmx, 2, sum)
	y<-(co/2)+apply(harmy, 2, sum)
	list(x=x, y=y)
}

#########################################################################
# Calculating cumulative power of harmonics (p. 228f)			#
# Necessary input variables:						#
#		an, bn, cn, dn: Set of coefficients per harmonic	#
#			*list*						#
#		first: Should the first harmonic be included in the...	#
#			calculations?					#
#			*logical*					#
#			TRUE=include first harmonic			#
#			FALSE=do not include first harmonic		#
#			default=FALSE					#
#		last: Last harmonic for which power should...		#
#			be calculated					#
#			*integer*					#
#			default=20					#
# Output data: list containing cumulative power of harmonics		#
# Input dataset: list of harmonics from elliptic Fourier analysis...	#
#		(can, but do not have to be normalized)			#
#########################################################################

Power<-function(an, bn, cn, dn, first=FALSE, last=20) {
	P<-(an^2+bn^2+cn^3+dn^2)/2
	{if (first==FALSE){PowerSum<-(cumsum(P[-1])/sum(P[-1]))[1:last]}
		else{PowerSum<-(cumsum(P)/sum(P))[1:last]}
	}
}

#########################################################################
# Averaging data from several measurements and calculating...		#
#	measurement error (pp. 63--67)					#
# Necessary input variables:						#
#		Objects: list of data set names containing harmonics	#
#			*vector* of *strings*				#
#		Centroids: do the last two columns contain centroid...	#
#			coordinates?					#
#			*logical*					#
#			TRUE: centroid coordinates contained in last...	#
#			two columns					#
#			FALSE: data contain coefficients only		#
#			default=TRUE					#
#		Output: name of output file with averaged Fourier...	#
#			coefficients to be exported			#
#			*string*					#
# Output data: File with average fourier coefficients, error...		#
#			calculation					#
# Input dataset: Files with elliptic Fourier coefficients, harmonics...	#
#			as sets of four coefficients in columns,...	#
#			specimens in rows, last two columns may...	#
#			contain centroid coordinates			#
#########################################################################

OutlineAverage<-function (Objects, Centroids=TRUE, Output) {
	#Read data into array
	Read.Obj<-function (X) {eval(parse(text=X))}
	TT<-Read.Obj(Objects[1])
	Dat<-array(NA, dim=c(dim(TT)[1], dim(TT)[2], length(Objects)))
	for (i in 1:(length(Objects))) {
		Dat[,,i]<-as.matrix(Read.Obj(Objects[i]))
	}
	
	#Calculate mean harmonics
	EFA.Mean<-apply(Dat, c(1, 2), mean)
	colnames(EFA.Mean)<-colnames(TT)
	rownames(EFA.Mean)<-rownames(TT)
	write.table(EFA.Mean, paste(Output, "MeanCoefficients.txt", sep="_"))
	
	#Eliminate centroid columns
	if (Centroids==TRUE) {DatFull<-Dat; Dat<-Dat[,-c(dim(Dat)[2]-1, dim(Dat)[2]),]}
	
	#Define factors
	Session.factor<-gl(dim(Dat)[3], (dim(Dat)[1]))
	Individual.factor<-as.factor(rep((1:dim(Dat)[1]), dim(Dat)[3]))
	
	#Calculate relative errors of replications
	RelErr<-list()
	for (i in 1:(dim(Dat)[2])) {
		Hm<-vector(length=0)
		for (j in 1:(dim(Dat)[3])) {
			Hm<-append(Hm, as.vector(t(Dat[,i,j])))
		}
		SE<-summary(aov(H~Session.factor))
		{if (SE[[1]][2,3]>SE[[1]][1,3]) {pSE<-1} else {pSE<-0}}
		ME<-summary(aov(H~Individual.factor))
		{if (ME[[1]][1,3]>=ME[[1]][2,3]) {pME<-1} else {pSE<-0}}
		s2within<-MSwithin<-ME[[1]][2,3]
		MSamong<-ME[[1]][1,3]
		s2among<-(MSamong-MSwithin)/2
		Err<-s2within/(s2within+s2among)*100
		{if (i%%4==1) {RelErr$F1$Sessionfactor<-append(RelErr$F1$Sessionfactor,pSE);RelErr$F1$Individualfactor<-append(RelErr$F1$Individualfactor,pME);RelErr$F1$RelativeError<-append(RelErr$F1$RelativeError,Err)}
		else if (i%%4==2) {RelErr$F2$Sessionfactor<-append(RelErr$F2$Sessionfactor,pSE);RelErr$F2$Individualfactor<-append(RelErr$F2$Individualfactor,pME);RelErr$F2$RelativeError<-append(RelErr$F2$RelativeError,Err)}
		else if (i%%4==3) {RelErr$F3$Sessionfactor<-append(RelErr$F3$Sessionfactor,pSE);RelErr$F3$Individualfactor<-append(RelErr$F3$Individualfactor,pME);RelErr$F3$RelativeError<-append(RelErr$F3$RelativeError,Err)}
		else if (i%%4==0) {RelErr$F4$Sessionfactor<-append(RelErr$F4$Sessionfactor,pSE);RelErr$F4$Individualfactor<-append(RelErr$F4$Individualfactor,pME);RelErr$F4$RelativeError<-append(RelErr$F4$RelativeError,Err)}}
	}
	
	#Export measurement error report
	sink(paste(Output, "ErrorReport.txt", sep="_"), type="output")
	writeLines("ERROR REPORT \nLists per coefficient type (F1--F4) per harmonic: \n-Whether the session factor (influence of replicated outline extraction) introduced a significant error: 1---no systematic error, 0---systematic error, consider exclusion/re-evaluation \n-Whether the individuals showed larger between-individual variance than within-individual variance (error): 1---between-individual variance dominates, 0---within-individual variance dominates, consider exclusion/re-evaluation \n-The relative measurement error (in %) after Yezerinac et al. (1992) Syst. Biol. 41: 471-482 \n")
	print(RelErr)
	sink()
}


#--------------------------------------------

#Examples

#Converting images
#ImageConversion("Foram", 1, 2, ".jpg", ".ppm")

#Extract outlines and write in files
#OutlineExtraction("Foram", 1, 2)

#Perform Zahn--Roskies Fourier analysis
#f1<-ZRfourier(Dat,40)
#if1<-iZRfourier(f1$ao,f1$an,f1$bn,40,100)
#plot(if1$X,if1$Y,type="l")

#Perform elliptic Fourier analysis
#Dat<-read.table("Foram1.txt", header=TRUE)
#layout(matrix((1:9),3,3))
#par=(mar=c(2,2,2,2))
#ef1<-efourier(Dat)
#ef2<-NEF(Dat)
#for (i in 1:9){
#	ief1<-iefourier(ef1$an, ef1$bn, ef1$cn, ef1$dn, i, 64, ef1$ao, ef1$co)
#	plot(Dat, type="l", asp=1, frame=FALSE, main=paste("Harmonics 0 to",i,sep=" "), col="grey")
#	polygon(Dat, col="grey", border=NA)
#	lines(ief1$x, ief1$y, type="l")
#}

#Calculate replication error
#setwd("C:/R_TestData/Outline")
#H1<-read.table("EllipticFourierCoefficients1.txt",header=TRUE,sep="\t")
#H2<-read.table("EllipticFourierCoefficients2.txt",header=TRUE,sep="\t")
#OutlineAverage(Objects=c("H1", "H2"), Centroids=TRUE, "Test")

#--------------------------------------------
#--------------------------------------------
#Version History
#1.0	Finished Program
#1.1	Adds option in OutlineExtraction to export the raw (i.e. unsmoothed) outlines as well
#1.2	Fixed a problem that would occur in the success table of the outline extraction, if StartNum was not 1
#1.3	Included function to automatically average over replicates and calculate measurement errors
#--------------------------------------------
#--------------------------------------------

